# pyright: reportConstantRedefinition=none
"""The lan commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2604B, SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B,
SMU2651A, SMU2657A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - lan.applysettings()
    - lan.autoconnect
    - lan.config.dns.address[N]
    - lan.config.dns.domain
    - lan.config.dns.dynamic
    - lan.config.dns.hostname
    - lan.config.dns.verify
    - lan.config.duplex
    - lan.config.gateway
    - lan.config.ipaddress
    - lan.config.method
    - lan.config.speed
    - lan.config.subnetmask
    - lan.linktimeout
    - lan.lxidomain
    - lan.nagle
    - lan.reset()
    - lan.restoredefaults()
    - lan.status.dns.address[N]
    - lan.status.dns.name
    - lan.status.duplex
    - lan.status.gateway
    - lan.status.ipaddress
    - lan.status.macaddress
    - lan.status.port.dst
    - lan.status.port.rawsocket
    - lan.status.port.telnet
    - lan.status.port.vxi11
    - lan.status.speed
    - lan.status.subnetmask
    - lan.timedwait
    - lan.trigger[N].assert()
    - lan.trigger[N].clear()
    - lan.trigger[N].connect()
    - lan.trigger[N].connected
    - lan.trigger[N].disconnect()
    - lan.trigger[N].ipaddress
    - lan.trigger[N].mode
    - lan.trigger[N].overrun
    - lan.trigger[N].protocol
    - lan.trigger[N].pseudostate
    - lan.trigger[N].stimulus
    - lan.trigger[N].wait()
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING, Union

from ..helpers import (
    BaseTSPCmd,
    DefaultDictDeviceCommunication,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class LanTriggerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``lan.trigger[N]`` command tree.

    Info:
        - ``N``, the LAN event number (1 to 8).

    Constants:
        - ``.EVENT_ID``: The event identifier used to route the LAN trigger to other subsystems
          (using stimulus properties).

    Properties and methods:
        - ``.assert()``: The ``lan.trigger[N].assert()`` function.
        - ``.clear()``: The ``lan.trigger[N].clear()`` function.
        - ``.connect()``: The ``lan.trigger[N].connect()`` function.
        - ``.connected``: The ``lan.trigger[N].connected`` attribute.
        - ``.disconnect()``: The ``lan.trigger[N].disconnect()`` function.
        - ``.ipaddress``: The ``lan.trigger[N].ipaddress`` attribute.
        - ``.mode``: The ``lan.trigger[N].mode`` attribute.
        - ``.overrun``: The ``lan.trigger[N].overrun`` attribute.
        - ``.protocol``: The ``lan.trigger[N].protocol`` attribute.
        - ``.pseudostate``: The ``lan.trigger[N].pseudostate`` attribute.
        - ``.stimulus``: The ``lan.trigger[N].stimulus`` attribute.
        - ``.wait()``: The ``lan.trigger[N].wait()`` function.
    """

    EVENT_ID = "lan.trigger[N].EVENT_ID"
    """str: The event identifier used to route the LAN trigger to other subsystems (using stimulus properties)."""  # noqa: E501

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )

    @property
    def connected(self) -> str:
        """Access the ``lan.trigger[N].connected`` attribute.

        Description:
            - This attribute stores the LAN event connection state.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].connected)`` query.

        TSP Syntax:
            ```
            - print(lan.trigger[N].connected)
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".connected"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.connected)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.connected`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def ipaddress(self) -> str:
        """Access the ``lan.trigger[N].ipaddress`` attribute.

        Description:
            - This attribute specifies the address (in dotted-decimal format) of UDP or TCP
              listeners.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].ipaddress)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].ipaddress = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].ipaddress = value
            - print(lan.trigger[N].ipaddress)
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".ipaddress"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.ipaddress)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.ipaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @ipaddress.setter
    def ipaddress(self, value: Union[str, float]) -> None:
        """Access the ``lan.trigger[N].ipaddress`` attribute.

        Description:
            - This attribute specifies the address (in dotted-decimal format) of UDP or TCP
              listeners.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].ipaddress)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].ipaddress = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].ipaddress = value
            - print(lan.trigger[N].ipaddress)
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".ipaddress", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.ipaddress = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.ipaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def mode(self) -> str:
        """Access the ``lan.trigger[N].mode`` attribute.

        Description:
            - This attribute sets the trigger operation and detection mode of the specified LAN
              event.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].mode = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].mode = value
            - print(lan.trigger[N].mode)
            ```

        Info:
            - ``N``, a number representing the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.mode)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``lan.trigger[N].mode`` attribute.

        Description:
            - This attribute sets the trigger operation and detection mode of the specified LAN
              event.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].mode = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].mode = value
            - print(lan.trigger[N].mode)
            ```

        Info:
            - ``N``, a number representing the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.mode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def overrun(self) -> str:
        """Access the ``lan.trigger[N].overrun`` attribute.

        Description:
            - This attribute contains the overrun status of the LAN event detector.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].overrun)`` query.

        TSP Syntax:
            ```
            - print(lan.trigger[N].overrun)
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".overrun"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.overrun)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.overrun`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def protocol(self) -> str:
        """Access the ``lan.trigger[N].protocol`` attribute.

        Description:
            - This attribute sets the LAN protocol to use for sending trigger messages.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].protocol)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].protocol = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].protocol = value
            - print(lan.trigger[N].protocol)
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".protocol"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.protocol)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.protocol`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @protocol.setter
    def protocol(self, value: Union[str, float]) -> None:
        """Access the ``lan.trigger[N].protocol`` attribute.

        Description:
            - This attribute sets the LAN protocol to use for sending trigger messages.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].protocol)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].protocol = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].protocol = value
            - print(lan.trigger[N].protocol)
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".protocol", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.protocol = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.protocol`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def pseudostate(self) -> str:
        """Access the ``lan.trigger[N].pseudostate`` attribute.

        Description:
            - This attribute sets the simulated line state for the LAN trigger.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].pseudostate)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].pseudostate = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].pseudostate = value
            - print(lan.trigger[N].pseudostate)
            ```

        Info:
            - ``N``, a number representing the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".pseudostate"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.pseudostate)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.pseudostate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @pseudostate.setter
    def pseudostate(self, value: Union[str, float]) -> None:
        """Access the ``lan.trigger[N].pseudostate`` attribute.

        Description:
            - This attribute sets the simulated line state for the LAN trigger.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].pseudostate)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].pseudostate = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].pseudostate = value
            - print(lan.trigger[N].pseudostate)
            ```

        Info:
            - ``N``, a number representing the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".pseudostate", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.pseudostate = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.pseudostate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def stimulus(self) -> str:
        """Access the ``lan.trigger[N].stimulus`` attribute.

        Description:
            - This attribute specifies events that cause this trigger to assert.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].stimulus = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].stimulus = value
            - print(lan.trigger[N].stimulus)
            ```

        Info:
            - ``N``, a number specifying the trigger packet over the LAN for which to set or query
              the trigger source (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``lan.trigger[N].stimulus`` attribute.

        Description:
            - This attribute specifies events that cause this trigger to assert.

        Usage:
            - Accessing this property will send the ``print(lan.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``lan.trigger[N].stimulus = value``
              command.

        TSP Syntax:
            ```
            - lan.trigger[N].stimulus = value
            - print(lan.trigger[N].stimulus)
            ```

        Info:
            - ``N``, a number specifying the trigger packet over the LAN for which to set or query
              the trigger source (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def assert_(self) -> None:
        """Run the ``lan.trigger[N].assert()`` function.

        Description:
            - This function simulates the occurrence of the trigger and generates the corresponding
              event ID.

        TSP Syntax:
            ```
            - lan.trigger[N].assert()
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.assert()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.assert()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``lan.trigger[N].clear()`` function.

        Description:
            - This function clears the event detector for a LAN trigger.

        TSP Syntax:
            ```
            - lan.trigger[N].clear()
            ```

        Info:
            - ``N``, the LAN event number (1 to 8) to clear.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.clear()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def connect(self) -> None:
        """Run the ``lan.trigger[N].connect()`` function.

        Description:
            - This function prepares the event generator for outgoing trigger events.

        TSP Syntax:
            ```
            - lan.trigger[N].connect()
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.connect()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.connect()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def disconnect(self) -> None:
        """Run the ``lan.trigger[N].disconnect()`` function.

        Description:
            - This function disconnects the LAN trigger.

        TSP Syntax:
            ```
            - lan.trigger[N].disconnect()
            ```

        Info:
            - ``N``, the LAN event number (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.disconnect()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.disconnect()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def wait(self, timeout: float) -> str:
        """Run the ``lan.trigger[N].wait()`` function.

        Description:
            - This function waits for an input trigger.

        TSP Syntax:
            ```
            - lan.trigger[N].wait()
            ```

        Args:
            timeout: Maximum amount of time in seconds to wait for the trigger event.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.wait({timeout}))"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.wait()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class LanStatusPort(BaseTSPCmd):
    """The ``lan.status.port`` command tree.

    Info:
        - ``port``, the dead socket termination socket port number.

    Properties and methods:
        - ``.dst``: The ``lan.status.port.dst`` attribute.
        - ``.rawsocket``: The ``lan.status.port.rawsocket`` attribute.
        - ``.telnet``: The ``lan.status.port.telnet`` attribute.
        - ``.vxi11``: The ``lan.status.port.vxi11`` attribute.
    """

    @property
    def dst(self) -> str:
        """Access the ``lan.status.port.dst`` attribute.

        Description:
            - This attribute contains the LAN dead socket termination port number.

        Usage:
            - Accessing this property will send the ``print(lan.status.port.dst)`` query.

        TSP Syntax:
            ```
            - print(lan.status.port.dst)
            ```

        Info:
            - ``port``, the dead socket termination socket port number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".dst"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.dst)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.dst`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def rawsocket(self) -> str:
        """Access the ``lan.status.port.rawsocket`` attribute.

        Description:
            - This attribute contains the LAN raw socket connection port number.

        Usage:
            - Accessing this property will send the ``print(lan.status.port.rawsocket)`` query.

        TSP Syntax:
            ```
            - print(lan.status.port.rawsocket)
            ```

        Info:
            - ``port``, the raw socket port number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".rawsocket"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.rawsocket)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.rawsocket`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def telnet(self) -> str:
        """Access the ``lan.status.port.telnet`` attribute.

        Description:
            - This attribute contains the LAN Telnet connection port number.

        Usage:
            - Accessing this property will send the ``print(lan.status.port.telnet)`` query.

        TSP Syntax:
            ```
            - print(lan.status.port.telnet)
            ```

        Info:
            - ``port``, the telnet port number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".telnet"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.telnet)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.telnet`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def vxi11(self) -> str:
        """Access the ``lan.status.port.vxi11`` attribute.

        Description:
            - This attribute contains the LAN VXI-11 connection port number.

        Usage:
            - Accessing this property will send the ``print(lan.status.port.vxi11)`` query.

        TSP Syntax:
            ```
            - print(lan.status.port.vxi11)
            ```

        Info:
            - ``port``, the lAN VXI-11 port number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".vxi11"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.vxi11)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.vxi11`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class LanStatusDns(BaseTSPCmd):
    """The ``lan.status.dns`` command tree.

    Properties and methods:
        - ``.address``: The ``lan.status.dns.address[N]`` attribute.
        - ``.name``: The ``lan.status.dns.name`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._address: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.address[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.address[{{key}}])",
            device=self._device,
        )

    @property
    def address(self) -> Dict[int, Union[str, float]]:
        """Access the ``lan.status.dns.address[N]`` attribute.

        Description:
            - This attribute contains the DNS server IP addresses.

        Usage:
            - Accessing an item from this property will send the
              ``print(lan.status.dns.address[N])`` query.

        TSP Syntax:
            ```
            - print(lan.status.dns.address[N])
            ```

        Info:
            - ``N``, the entry index (1, 2, or 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._address

    @property
    def name(self) -> str:
        """Access the ``lan.status.dns.name`` attribute.

        Description:
            - This attribute contains the present DNS fully qualified host name.

        Usage:
            - Accessing this property will send the ``print(lan.status.dns.name)`` query.

        TSP Syntax:
            ```
            - print(lan.status.dns.name)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".name"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.name)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.name`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class LanStatus(BaseTSPCmd):
    """The ``lan.status`` command tree.

    Properties and methods:
        - ``.dns``: The ``lan.status.dns`` command tree.
        - ``.duplex``: The ``lan.status.duplex`` attribute.
        - ``.gateway``: The ``lan.status.gateway`` attribute.
        - ``.ipaddress``: The ``lan.status.ipaddress`` attribute.
        - ``.macaddress``: The ``lan.status.macaddress`` attribute.
        - ``.port``: The ``lan.status.port`` command tree.
        - ``.speed``: The ``lan.status.speed`` attribute.
        - ``.subnetmask``: The ``lan.status.subnetmask`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._dns = LanStatusDns(device, f"{self._cmd_syntax}.dns")
        self._port = LanStatusPort(device, f"{self._cmd_syntax}.port")

    @property
    def dns(self) -> LanStatusDns:
        """Return the ``lan.status.dns`` command tree.

        Sub-properties and sub-methods:
            - ``.address``: The ``lan.status.dns.address[N]`` attribute.
            - ``.name``: The ``lan.status.dns.name`` attribute.
        """
        return self._dns

    @property
    def duplex(self) -> str:
        """Access the ``lan.status.duplex`` attribute.

        Description:
            - This attribute contains the duplex mode presently in use by the LAN interface.

        Usage:
            - Accessing this property will send the ``print(lan.status.duplex)`` query.

        TSP Syntax:
            ```
            - print(lan.status.duplex)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".duplex"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.duplex)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.duplex`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def gateway(self) -> str:
        """Access the ``lan.status.gateway`` attribute.

        Description:
            - This attribute contains the gateway address presently in use by the LAN interface.

        Usage:
            - Accessing this property will send the ``print(lan.status.gateway)`` query.

        TSP Syntax:
            ```
            - print(lan.status.gateway)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".gateway"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.gateway)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.gateway`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def ipaddress(self) -> str:
        """Access the ``lan.status.ipaddress`` attribute.

        Description:
            - This attribute contains the LAN IP address presently in use by the LAN interface.

        Usage:
            - Accessing this property will send the ``print(lan.status.ipaddress)`` query.

        TSP Syntax:
            ```
            - print(lan.status.ipaddress)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".ipaddress"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.ipaddress)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.ipaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def macaddress(self) -> str:
        """Access the ``lan.status.macaddress`` attribute.

        Description:
            - This attribute contains the LAN MAC address.

        Usage:
            - Accessing this property will send the ``print(lan.status.macaddress)`` query.

        TSP Syntax:
            ```
            - print(lan.status.macaddress)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".macaddress"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.macaddress)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.macaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def port(self) -> LanStatusPort:
        """Return the ``lan.status.port`` command tree.

        Info:
            - ``port``, the dead socket termination socket port number.

        Sub-properties and sub-methods:
            - ``.dst``: The ``lan.status.port.dst`` attribute.
            - ``.rawsocket``: The ``lan.status.port.rawsocket`` attribute.
            - ``.telnet``: The ``lan.status.port.telnet`` attribute.
            - ``.vxi11``: The ``lan.status.port.vxi11`` attribute.
        """
        return self._port

    @property
    def speed(self) -> str:
        """Access the ``lan.status.speed`` attribute.

        Description:
            - This attribute contains the LAN speed.

        Usage:
            - Accessing this property will send the ``print(lan.status.speed)`` query.

        TSP Syntax:
            ```
            - print(lan.status.speed)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".speed"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.speed)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.speed`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def subnetmask(self) -> str:
        """Access the ``lan.status.subnetmask`` attribute.

        Description:
            - This attribute contains the LAN subnet mask that is presently in use by the LAN
              interface.

        Usage:
            - Accessing this property will send the ``print(lan.status.subnetmask)`` query.

        TSP Syntax:
            ```
            - print(lan.status.subnetmask)
            ```

        Info:
            - ``mask``, a string specifying the subnet mask in dotted decimal notation.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".subnetmask"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.subnetmask)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.subnetmask`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class LanConfigDns(BaseTSPCmd):
    """The ``lan.config.dns`` command tree.

    Properties and methods:
        - ``.address``: The ``lan.config.dns.address[N]`` attribute.
        - ``.domain``: The ``lan.config.dns.domain`` attribute.
        - ``.dynamic``: The ``lan.config.dns.dynamic`` attribute.
        - ``.hostname``: The ``lan.config.dns.hostname`` attribute.
        - ``.verify``: The ``lan.config.dns.verify`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._address: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.address[{{key}}]",
            write_syntax=f"{self._cmd_syntax}.address[{{key}}] = ",
            query_syntax=f"print({self._cmd_syntax}.address[{{key}}])",
            device=self._device,
        )

    @property
    def address(self) -> Dict[int, Union[str, float]]:
        """Access the ``lan.config.dns.address[N]`` attribute.

        Description:
            - Configures DNS server IP addresses.

        Usage:
            - Accessing an item from this property will send the
              ``print(lan.config.dns.address[N])`` query.
            - Setting an item from this property to a value will send the
              ``lan.config.dns.address[N] = value`` command.

        TSP Syntax:
            ```
            - lan.config.dns.address[N] = value
            - print(lan.config.dns.address[N])
            ```

        Info:
            - ``N``, the entry index (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._address

    @property
    def domain(self) -> str:
        """Access the ``lan.config.dns.domain`` attribute.

        Description:
            - Configures the dynamic DNS domain.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.domain)`` query.
            - Setting this property to a value will send the ``lan.config.dns.domain = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.domain = value
            - print(lan.config.dns.domain)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".domain"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.domain)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.domain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @domain.setter
    def domain(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.dns.domain`` attribute.

        Description:
            - Configures the dynamic DNS domain.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.domain)`` query.
            - Setting this property to a value will send the ``lan.config.dns.domain = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.domain = value
            - print(lan.config.dns.domain)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".domain", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.domain = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.domain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def dynamic(self) -> str:
        """Access the ``lan.config.dns.dynamic`` attribute.

        Description:
            - Enables or disables the dynamic DNS registration.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.dynamic)`` query.
            - Setting this property to a value will send the ``lan.config.dns.dynamic = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.dynamic = value
            - print(lan.config.dns.dynamic)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".dynamic"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.dynamic)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.dynamic`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @dynamic.setter
    def dynamic(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.dns.dynamic`` attribute.

        Description:
            - Enables or disables the dynamic DNS registration.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.dynamic)`` query.
            - Setting this property to a value will send the ``lan.config.dns.dynamic = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.dynamic = value
            - print(lan.config.dns.dynamic)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".dynamic", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.dynamic = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.dynamic`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def hostname(self) -> str:
        """Access the ``lan.config.dns.hostname`` attribute.

        Description:
            - This attribute defines the dynamic DNS host name.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.hostname)`` query.
            - Setting this property to a value will send the ``lan.config.dns.hostname = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.hostname = value
            - print(lan.config.dns.hostname)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".hostname"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.hostname)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.hostname`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @hostname.setter
    def hostname(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.dns.hostname`` attribute.

        Description:
            - This attribute defines the dynamic DNS host name.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.hostname)`` query.
            - Setting this property to a value will send the ``lan.config.dns.hostname = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.hostname = value
            - print(lan.config.dns.hostname)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".hostname", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.hostname = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.hostname`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def verify(self) -> str:
        """Access the ``lan.config.dns.verify`` attribute.

        Description:
            - This attribute defines the DNS host name verification state.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.verify)`` query.
            - Setting this property to a value will send the ``lan.config.dns.verify = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.verify = value
            - print(lan.config.dns.verify)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".verify"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.verify)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.verify`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @verify.setter
    def verify(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.dns.verify`` attribute.

        Description:
            - This attribute defines the DNS host name verification state.

        Usage:
            - Accessing this property will send the ``print(lan.config.dns.verify)`` query.
            - Setting this property to a value will send the ``lan.config.dns.verify = value``
              command.

        TSP Syntax:
            ```
            - lan.config.dns.verify = value
            - print(lan.config.dns.verify)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".verify", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.verify = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.verify`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class LanConfig(BaseTSPCmd):
    """The ``lan.config`` command tree.

    Properties and methods:
        - ``.dns``: The ``lan.config.dns`` command tree.
        - ``.duplex``: The ``lan.config.duplex`` attribute.
        - ``.gateway``: The ``lan.config.gateway`` attribute.
        - ``.ipaddress``: The ``lan.config.ipaddress`` attribute.
        - ``.method``: The ``lan.config.method`` attribute.
        - ``.speed``: The ``lan.config.speed`` attribute.
        - ``.subnetmask``: The ``lan.config.subnetmask`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._dns = LanConfigDns(device, f"{self._cmd_syntax}.dns")

    @property
    def dns(self) -> LanConfigDns:
        """Return the ``lan.config.dns`` command tree.

        Sub-properties and sub-methods:
            - ``.address``: The ``lan.config.dns.address[N]`` attribute.
            - ``.domain``: The ``lan.config.dns.domain`` attribute.
            - ``.dynamic``: The ``lan.config.dns.dynamic`` attribute.
            - ``.hostname``: The ``lan.config.dns.hostname`` attribute.
            - ``.verify``: The ``lan.config.dns.verify`` attribute.
        """
        return self._dns

    @property
    def duplex(self) -> str:
        """Access the ``lan.config.duplex`` attribute.

        Description:
            - This attribute defines the LAN duplex mode.

        Usage:
            - Accessing this property will send the ``print(lan.config.duplex)`` query.
            - Setting this property to a value will send the ``lan.config.duplex = value`` command.

        TSP Syntax:
            ```
            - lan.config.duplex = value
            - print(lan.config.duplex)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".duplex"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.duplex)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.duplex`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @duplex.setter
    def duplex(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.duplex`` attribute.

        Description:
            - This attribute defines the LAN duplex mode.

        Usage:
            - Accessing this property will send the ``print(lan.config.duplex)`` query.
            - Setting this property to a value will send the ``lan.config.duplex = value`` command.

        TSP Syntax:
            ```
            - lan.config.duplex = value
            - print(lan.config.duplex)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".duplex", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.duplex = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.duplex`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def gateway(self) -> str:
        """Access the ``lan.config.gateway`` attribute.

        Description:
            - This attribute contains the LAN default gateway address.

        Usage:
            - Accessing this property will send the ``print(lan.config.gateway)`` query.
            - Setting this property to a value will send the ``lan.config.gateway = value`` command.

        TSP Syntax:
            ```
            - lan.config.gateway = value
            - print(lan.config.gateway)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".gateway"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.gateway)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.gateway`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @gateway.setter
    def gateway(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.gateway`` attribute.

        Description:
            - This attribute contains the LAN default gateway address.

        Usage:
            - Accessing this property will send the ``print(lan.config.gateway)`` query.
            - Setting this property to a value will send the ``lan.config.gateway = value`` command.

        TSP Syntax:
            ```
            - lan.config.gateway = value
            - print(lan.config.gateway)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".gateway", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.gateway = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.gateway`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def ipaddress(self) -> str:
        """Access the ``lan.config.ipaddress`` attribute.

        Description:
            - This command specifies the LAN IP address.

        Usage:
            - Accessing this property will send the ``print(lan.config.ipaddress)`` query.
            - Setting this property to a value will send the ``lan.config.ipaddress = value``
              command.

        TSP Syntax:
            ```
            - lan.config.ipaddress = value
            - print(lan.config.ipaddress)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".ipaddress"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.ipaddress)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.ipaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @ipaddress.setter
    def ipaddress(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.ipaddress`` attribute.

        Description:
            - This command specifies the LAN IP address.

        Usage:
            - Accessing this property will send the ``print(lan.config.ipaddress)`` query.
            - Setting this property to a value will send the ``lan.config.ipaddress = value``
              command.

        TSP Syntax:
            ```
            - lan.config.ipaddress = value
            - print(lan.config.ipaddress)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".ipaddress", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.ipaddress = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.ipaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def method(self) -> str:
        """Access the ``lan.config.method`` attribute.

        Description:
            - This attribute contains the LAN settings configuration method.

        Usage:
            - Accessing this property will send the ``print(lan.config.method)`` query.
            - Setting this property to a value will send the ``lan.config.method = value`` command.

        TSP Syntax:
            ```
            - lan.config.method = value
            - print(lan.config.method)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".method"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.method)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.method`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @method.setter
    def method(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.method`` attribute.

        Description:
            - This attribute contains the LAN settings configuration method.

        Usage:
            - Accessing this property will send the ``print(lan.config.method)`` query.
            - Setting this property to a value will send the ``lan.config.method = value`` command.

        TSP Syntax:
            ```
            - lan.config.method = value
            - print(lan.config.method)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".method", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.method = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.method`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def speed(self) -> str:
        """Access the ``lan.config.speed`` attribute.

        Description:
            - This attribute contains the LAN speed used when restarting in manual configuration
              mode.

        Usage:
            - Accessing this property will send the ``print(lan.config.speed)`` query.
            - Setting this property to a value will send the ``lan.config.speed = value`` command.

        TSP Syntax:
            ```
            - lan.config.speed = value
            - print(lan.config.speed)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".speed"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.speed)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.speed`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @speed.setter
    def speed(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.speed`` attribute.

        Description:
            - This attribute contains the LAN speed used when restarting in manual configuration
              mode.

        Usage:
            - Accessing this property will send the ``print(lan.config.speed)`` query.
            - Setting this property to a value will send the ``lan.config.speed = value`` command.

        TSP Syntax:
            ```
            - lan.config.speed = value
            - print(lan.config.speed)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".speed", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.speed = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.speed`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def subnetmask(self) -> str:
        """Access the ``lan.config.subnetmask`` attribute.

        Description:
            - This attribute contains the LAN subnet mask.

        Usage:
            - Accessing this property will send the ``print(lan.config.subnetmask)`` query.
            - Setting this property to a value will send the ``lan.config.subnetmask = value``
              command.

        TSP Syntax:
            ```
            - lan.config.subnetmask = value
            - print(lan.config.subnetmask)
            ```

        Info:
            - ``mask``, the string that specifies the LAN subnet mask value in dotted decimal
              notation.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".subnetmask"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.subnetmask)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.subnetmask`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @subnetmask.setter
    def subnetmask(self, value: Union[str, float]) -> None:
        """Access the ``lan.config.subnetmask`` attribute.

        Description:
            - This attribute contains the LAN subnet mask.

        Usage:
            - Accessing this property will send the ``print(lan.config.subnetmask)`` query.
            - Setting this property to a value will send the ``lan.config.subnetmask = value``
              command.

        TSP Syntax:
            ```
            - lan.config.subnetmask = value
            - print(lan.config.subnetmask)
            ```

        Info:
            - ``mask``, the string that specifies the LAN subnet mask value in dotted decimal
              notation.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".subnetmask", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.subnetmask = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.subnetmask`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Lan(BaseTSPCmd):
    """The ``lan`` command tree.

    Constants:
        - ``.AUTO``: Selects automatic sequencing of configuration.
        - ``.DISABLE``: Disables automatic link reconnection and monitoring.
        - ``.ENABLE``: Enables automatic link reconnection and monitoring.
        - ``.FULL``: Full-duplex operation.
        - ``.HALF``: Half-duplex operation.
        - ``.MANUAL``: Use only manually specified configuration settings.
        - ``.MULTICAST``: Sets the LAN protocol to use for sending trigger messages to multicast.
        - ``.TCP``: Use TCP protocol.
        - ``.TRIG_EITHER``: Sets the trigger operation and detection mode of the specified LAN event
          to rising or falling edge (positive or negative state).
        - ``.TRIG_FALLING``: Sets the trigger operation and detection mode of the specified LAN
          event to falling edge (negative state).
        - ``.TRIG_RISING``: Sets the trigger operation and detection mode of the specified LAN event
          to rising edge (positive state).
        - ``.TRIG_RISINGA``: Sets the trigger operation and detection mode of the specified LAN
          event to rising edge (positive state).
        - ``.TRIG_RISINGM``: Sets the trigger operation and detection mode of the specified LAN
          event to rising edge (positive state).
        - ``.TRIG_SYNCHRONOUS``: Use for compatibility with older instruments; sets the trigger
          operation and detection mode of the specified LAN event to falling edge (negative state).
        - ``.TRIG_SYNCHRONOUSA``: Sets the trigger operation and detection mode of the specified LAN
          event to falling edge (negative state).
        - ``.TRIG_SYNCHRONOUSM``: Sets the trigger operation and detection mode of the specified LAN
          event to rising edge (positive state).
        - ``.UDP``: Use UDP protocol.

    Properties and methods:
        - ``.applysettings()``: The ``lan.applysettings()`` function.
        - ``.autoconnect``: The ``lan.autoconnect`` attribute.
        - ``.config``: The ``lan.config`` command tree.
        - ``.linktimeout``: The ``lan.linktimeout`` attribute.
        - ``.lxidomain``: The ``lan.lxidomain`` attribute.
        - ``.nagle``: The ``lan.nagle`` attribute.
        - ``.reset()``: The ``lan.reset()`` function.
        - ``.restoredefaults()``: The ``lan.restoredefaults()`` function.
        - ``.status``: The ``lan.status`` command tree.
        - ``.timedwait``: The ``lan.timedwait`` attribute.
        - ``.trigger``: The ``lan.trigger[N]`` command tree.
    """

    AUTO = "lan.AUTO"
    """str: Selects automatic sequencing of configuration."""
    DISABLE = "lan.DISABLE"
    """str: Disables automatic link reconnection and monitoring."""
    ENABLE = "lan.ENABLE"
    """str: Enables automatic link reconnection and monitoring."""
    FULL = "lan.FULL"
    """str: Full-duplex operation."""
    HALF = "lan.HALF"
    """str: Half-duplex operation."""
    MANUAL = "lan.MANUAL"
    """str: Use only manually specified configuration settings."""
    MULTICAST = "lan.MULTICAST"
    """str: Sets the LAN protocol to use for sending trigger messages to multicast."""
    TCP = "lan.TCP"
    """str: Use TCP protocol."""
    TRIG_EITHER = "lan.TRIG_EITHER"
    """str: Sets the trigger operation and detection mode of the specified LAN event to rising or falling edge (positive or negative state)."""  # noqa: E501
    TRIG_FALLING = "lan.TRIG_FALLING"
    """str: Sets the trigger operation and detection mode of the specified LAN event to falling edge (negative state)."""  # noqa: E501
    TRIG_RISING = "lan.TRIG_RISING"
    """str: Sets the trigger operation and detection mode of the specified LAN event to rising edge (positive state)."""  # noqa: E501
    TRIG_RISINGA = "lan.TRIG_RISINGA"
    """str: Sets the trigger operation and detection mode of the specified LAN event to rising edge (positive state)."""  # noqa: E501
    TRIG_RISINGM = "lan.TRIG_RISINGM"
    """str: Sets the trigger operation and detection mode of the specified LAN event to rising edge (positive state)."""  # noqa: E501
    TRIG_SYNCHRONOUS = "lan.TRIG_SYNCHRONOUS"
    """str: Use for compatibility with older instruments; sets the trigger operation and detection mode of the specified LAN event to falling edge (negative state)."""  # noqa: E501
    TRIG_SYNCHRONOUSA = "lan.TRIG_SYNCHRONOUSA"
    """str: Sets the trigger operation and detection mode of the specified LAN event to falling edge (negative state)."""  # noqa: E501
    TRIG_SYNCHRONOUSM = "lan.TRIG_SYNCHRONOUSM"
    """str: Sets the trigger operation and detection mode of the specified LAN event to rising edge (positive state)."""  # noqa: E501
    UDP = "lan.UDP"
    """str: Use UDP protocol."""

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "lan") -> None:
        super().__init__(device, cmd_syntax)
        self._config = LanConfig(device, f"{self._cmd_syntax}.config")
        self._status = LanStatus(device, f"{self._cmd_syntax}.status")
        self._trigger: Dict[int, LanTriggerItem] = DefaultDictPassKeyToFactory(
            lambda x: LanTriggerItem(device, f"{self._cmd_syntax}.trigger[{x}]")
        )

    @property
    def autoconnect(self) -> str:
        """Access the ``lan.autoconnect`` attribute.

        Description:
            - This attribute is used to enable or disable link monitoring.

        Usage:
            - Accessing this property will send the ``print(lan.autoconnect)`` query.
            - Setting this property to a value will send the ``lan.autoconnect = value`` command.

        TSP Syntax:
            ```
            - lan.autoconnect = value
            - print(lan.autoconnect)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".autoconnect"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.autoconnect)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.autoconnect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @autoconnect.setter
    def autoconnect(self, value: Union[str, float]) -> None:
        """Access the ``lan.autoconnect`` attribute.

        Description:
            - This attribute is used to enable or disable link monitoring.

        Usage:
            - Accessing this property will send the ``print(lan.autoconnect)`` query.
            - Setting this property to a value will send the ``lan.autoconnect = value`` command.

        TSP Syntax:
            ```
            - lan.autoconnect = value
            - print(lan.autoconnect)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".autoconnect", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.autoconnect = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.autoconnect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def config(self) -> LanConfig:
        """Return the ``lan.config`` command tree.

        Sub-properties and sub-methods:
            - ``.dns``: The ``lan.config.dns`` command tree.
            - ``.duplex``: The ``lan.config.duplex`` attribute.
            - ``.gateway``: The ``lan.config.gateway`` attribute.
            - ``.ipaddress``: The ``lan.config.ipaddress`` attribute.
            - ``.method``: The ``lan.config.method`` attribute.
            - ``.speed``: The ``lan.config.speed`` attribute.
            - ``.subnetmask``: The ``lan.config.subnetmask`` attribute.
        """
        return self._config

    @property
    def linktimeout(self) -> str:
        """Access the ``lan.linktimeout`` attribute.

        Description:
            - This attribute contains the LAN link timeout period.

        Usage:
            - Accessing this property will send the ``print(lan.linktimeout)`` query.
            - Setting this property to a value will send the ``lan.linktimeout = value`` command.

        TSP Syntax:
            ```
            - lan.linktimeout = value
            - print(lan.linktimeout)
            ```

        Info:
            - ``timeout``, the LAN link monitor time-out period (in seconds).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".linktimeout"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.linktimeout)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.linktimeout`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @linktimeout.setter
    def linktimeout(self, value: Union[str, float]) -> None:
        """Access the ``lan.linktimeout`` attribute.

        Description:
            - This attribute contains the LAN link timeout period.

        Usage:
            - Accessing this property will send the ``print(lan.linktimeout)`` query.
            - Setting this property to a value will send the ``lan.linktimeout = value`` command.

        TSP Syntax:
            ```
            - lan.linktimeout = value
            - print(lan.linktimeout)
            ```

        Info:
            - ``timeout``, the LAN link monitor time-out period (in seconds).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".linktimeout", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.linktimeout = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.linktimeout`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def lxidomain(self) -> str:
        """Access the ``lan.lxidomain`` attribute.

        Description:
            - This attribute contains the LXI domain.

        Usage:
            - Accessing this property will send the ``print(lan.lxidomain)`` query.
            - Setting this property to a value will send the ``lan.lxidomain = value`` command.

        TSP Syntax:
            ```
            - lan.lxidomain = value
            - print(lan.lxidomain)
            ```

        Info:
            - ``domain``, the LXI domain number (0 to 255).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".lxidomain"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.lxidomain)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.lxidomain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @lxidomain.setter
    def lxidomain(self, value: Union[str, float]) -> None:
        """Access the ``lan.lxidomain`` attribute.

        Description:
            - This attribute contains the LXI domain.

        Usage:
            - Accessing this property will send the ``print(lan.lxidomain)`` query.
            - Setting this property to a value will send the ``lan.lxidomain = value`` command.

        TSP Syntax:
            ```
            - lan.lxidomain = value
            - print(lan.lxidomain)
            ```

        Info:
            - ``domain``, the LXI domain number (0 to 255).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".lxidomain", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.lxidomain = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.lxidomain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def nagle(self) -> str:
        """Access the ``lan.nagle`` attribute.

        Description:
            - This attribute controls the state of the LAN Nagle algorithm.

        Usage:
            - Accessing this property will send the ``print(lan.nagle)`` query.
            - Setting this property to a value will send the ``lan.nagle = value`` command.

        TSP Syntax:
            ```
            - lan.nagle = value
            - print(lan.nagle)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".nagle"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.nagle)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.nagle`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @nagle.setter
    def nagle(self, value: Union[str, float]) -> None:
        """Access the ``lan.nagle`` attribute.

        Description:
            - This attribute controls the state of the LAN Nagle algorithm.

        Usage:
            - Accessing this property will send the ``print(lan.nagle)`` query.
            - Setting this property to a value will send the ``lan.nagle = value`` command.

        TSP Syntax:
            ```
            - lan.nagle = value
            - print(lan.nagle)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".nagle", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.nagle = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.nagle`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def status(self) -> LanStatus:
        """Return the ``lan.status`` command tree.

        Sub-properties and sub-methods:
            - ``.dns``: The ``lan.status.dns`` command tree.
            - ``.duplex``: The ``lan.status.duplex`` attribute.
            - ``.gateway``: The ``lan.status.gateway`` attribute.
            - ``.ipaddress``: The ``lan.status.ipaddress`` attribute.
            - ``.macaddress``: The ``lan.status.macaddress`` attribute.
            - ``.port``: The ``lan.status.port`` command tree.
            - ``.speed``: The ``lan.status.speed`` attribute.
            - ``.subnetmask``: The ``lan.status.subnetmask`` attribute.
        """
        return self._status

    @property
    def timedwait(self) -> str:
        """Access the ``lan.timedwait`` attribute.

        Description:
            - This attribute contains the LAN timed-wait state interval.

        Usage:
            - Accessing this property will send the ``print(lan.timedwait)`` query.
            - Setting this property to a value will send the ``lan.timedwait = value`` command.

        TSP Syntax:
            ```
            - lan.timedwait = value
            - print(lan.timedwait)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".timedwait"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.timedwait)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.timedwait`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @timedwait.setter
    def timedwait(self, value: Union[str, float]) -> None:
        """Access the ``lan.timedwait`` attribute.

        Description:
            - This attribute contains the LAN timed-wait state interval.

        Usage:
            - Accessing this property will send the ``print(lan.timedwait)`` query.
            - Setting this property to a value will send the ``lan.timedwait = value`` command.

        TSP Syntax:
            ```
            - lan.timedwait = value
            - print(lan.timedwait)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".timedwait", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.timedwait = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.timedwait`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def trigger(self) -> Dict[int, LanTriggerItem]:
        """Return the ``lan.trigger[N]`` command tree.

        Info:
            - ``N``, the LAN event number (1 to 8).

        Constants:
            - ``.EVENT_ID``: The event identifier used to route the LAN trigger to other subsystems
              (using stimulus properties).

        Sub-properties and sub-methods:
            - ``.assert()``: The ``lan.trigger[N].assert()`` function.
            - ``.clear()``: The ``lan.trigger[N].clear()`` function.
            - ``.connect()``: The ``lan.trigger[N].connect()`` function.
            - ``.connected``: The ``lan.trigger[N].connected`` attribute.
            - ``.disconnect()``: The ``lan.trigger[N].disconnect()`` function.
            - ``.ipaddress``: The ``lan.trigger[N].ipaddress`` attribute.
            - ``.mode``: The ``lan.trigger[N].mode`` attribute.
            - ``.overrun``: The ``lan.trigger[N].overrun`` attribute.
            - ``.protocol``: The ``lan.trigger[N].protocol`` attribute.
            - ``.pseudostate``: The ``lan.trigger[N].pseudostate`` attribute.
            - ``.stimulus``: The ``lan.trigger[N].stimulus`` attribute.
            - ``.wait()``: The ``lan.trigger[N].wait()`` function.
        """
        return self._trigger

    def applysettings(self) -> None:
        """Run the ``lan.applysettings()`` function.

        Description:
            - This function re-initializes the LAN interface with new settings.

        TSP Syntax:
            ```
            - lan.applysettings()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.applysettings()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.applysettings()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``lan.reset()`` function.

        Description:
            - This function resets the LAN interface.

        TSP Syntax:
            ```
            - lan.reset()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.reset()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def restoredefaults(self) -> None:
        """Run the ``lan.restoredefaults()`` function.

        Description:
            - This function resets LAN settings to default values.

        TSP Syntax:
            ```
            - lan.restoredefaults()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.restoredefaults()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.restoredefaults()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The MP5103 commands module.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""

from typing import Dict, Optional

from tm_devices.driver_mixins.device_control.tsp_control import TSPControl
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813

from .gen_3skc3w_mp.bit import Bit
from .gen_3skc3w_mp.digio import Digio
from .gen_3skc3w_mp.eventlog import Eventlog
from .gen_3skc3w_mp.firmware import Firmware
from .gen_3skc3w_mp.format import Format
from .gen_3skc3w_mp.fs import Fs
from .gen_3skc3w_mp.lan import Lan
from .gen_3skc3w_mp.localnode import Localnode
from .gen_3skc3w_mp.node import NodeItem
from .gen_3skc3w_mp.os import Os
from .gen_3skc3w_mp.script import Script
from .gen_3skc3w_mp.slot import SlotItem
from .gen_3skc3w_mp.status import Status
from .gen_3skc3w_mp.timer import Timer
from .gen_3skc3w_mp.trigger import Trigger
from .gen_3skc3w_mp.tsplink import Tsplink
from .gen_3skc3w_mp.tspnet import Tspnet
from .gen_3skc3w_mp.usbtmc import Usbtmc
from .gen_3skc3w_mp.user import User
from .gen_4ajvxe_mpmpsu.buffervar import Buffervar
from .gen_fnz1de_mpsmuss.io import Io
from .gen_fs14fx_daqdmmmpsmuss.dataqueue import Dataqueue
from .gen_fs14fx_daqdmmmpsmuss.userstring import Userstring
from .gen_fualds_mpsmu.scriptvar import Scriptvar
from .helpers import DefaultDictPassKeyToFactory, NoDeviceProvidedError


# pylint: disable=too-few-public-methods
class MP5103CommandConstants:
    """The MP5103 command argument constants.

    This provides access to all the string constants which can be used as arguments for MP5103
    commands.
    """

    DATAQUEUE_CAPACITY = "dataqueue.CAPACITY"
    DIGIO_TRIGGER_N_EVENT_ID = "digio.trigger[N].EVENT_ID"
    TRIGGER_EVENT_ID = "trigger.EVENT_ID"
    TRIGGER_EVENT_NONE = "trigger.EVENT_NONE"
    TRIGGER_GENERATOR_N_EVENT_ID = "trigger.generator[N].EVENT_ID"
    TRIGGER_TIMER_N_EVENT_ID = "trigger.timer[N].EVENT_ID"
    TSPLINK_TRIGGER_N_EVENT_ID = "tsplink.trigger[N].EVENT_ID"


#  pylint: disable=too-many-instance-attributes,too-many-public-methods
class MP5103Commands:
    """The MP5103 commands.

    This provides access to all the commands for the MP5103 device. See the documentation of each
    property for more usage information.

    Properties and methods:
        - ``.bit``: The ``bit`` command tree.
        - ``.buffer_var``: The ``bufferVar`` command tree.
        - ``.dataqueue``: The ``dataqueue`` command tree.
        - ``.delay()``: The ``delay()`` function.
        - ``.digio``: The ``digio`` command tree.
        - ``.eventlog``: The ``eventlog`` command tree.
        - ``.exit()``: The ``exit()`` function.
        - ``.firmware``: The ``firmware`` command tree.
        - ``.format``: The ``format`` command tree.
        - ``.fs``: The ``fs`` command tree.
        - ``.gettimezone()``: The ``gettimezone()`` function.
        - ``.io``: The ``io`` command tree.
        - ``.lan``: The ``lan`` command tree.
        - ``.localnode``: The ``localnode`` command tree.
        - ``.login()``: The ``login()`` function.
        - ``.logout()``: The ``logout()`` function.
        - ``.makegetter()``: The ``makegetter()`` function.
        - ``.makesetter()``: The ``makesetter()`` function.
        - ``.node``: The ``node[N]`` command tree.
        - ``.opc()``: The ``opc()`` function.
        - ``.os``: The ``os`` command tree.
        - ``.print()``: The ``print()`` function.
        - ``.printbuffer()``: The ``printbuffer()`` function.
        - ``.printnumber()``: The ``printnumber()`` function.
        - ``.reset()``: The ``reset()`` function.
        - ``.script_var``: The ``scriptVar`` command tree.
        - ``.script``: The ``script`` command tree.
        - ``.settime()``: The ``settime()`` function.
        - ``.settimezone()``: The ``settimezone()`` function.
        - ``.slot``: The ``slot[Z]`` command tree.
        - ``.status``: The ``status`` command tree.
        - ``.timer``: The ``timer`` command tree.
        - ``.trigger``: The ``trigger`` command tree.
        - ``.tsplink``: The ``tsplink`` command tree.
        - ``.tspnet``: The ``tspnet`` command tree.
        - ``.usbtmc``: The ``usbtmc`` command tree.
        - ``.user``: The ``user`` command tree.
        - ``.userstring``: The ``userstring`` command tree.
        - ``.waitcomplete()``: The ``waitcomplete()`` function.
    """

    def __init__(self, device: Optional[TSPControl] = None) -> None:
        self._device = device
        self._bit = Bit(device)
        self._buffer_var: Dict[str, Buffervar] = DefaultDictPassKeyToFactory(
            lambda x: Buffervar(device, str(x))
        )
        self._dataqueue = Dataqueue(device)
        self._digio = Digio(device)
        self._eventlog = Eventlog(device)
        self._firmware = Firmware(device)
        self._format = Format(device)
        self._fs = Fs(device)
        self._io = Io(device)
        self._lan = Lan(device)
        self._localnode = Localnode(device)
        self._node: Dict[int, NodeItem] = DefaultDictPassKeyToFactory(
            lambda x: NodeItem(device, f"node[{x}]")
        )
        self._os = Os(device)
        self._script = Script(device)
        self._script_var: Dict[str, Scriptvar] = DefaultDictPassKeyToFactory(
            lambda x: Scriptvar(device, str(x))
        )
        self._slot: Dict[int, SlotItem] = DefaultDictPassKeyToFactory(
            lambda x: SlotItem(device, f"slot[{x}]")
        )
        self._status = Status(device)
        self._timer = Timer(device)
        self._trigger = Trigger(device)
        self._tsplink = Tsplink(device)
        self._tspnet = Tspnet(device)
        self._usbtmc = Usbtmc(device)
        self._user = User(device)
        self._userstring = Userstring(device)

    @property
    def bit(self) -> Bit:
        """Return the ``bit`` command tree.

        Sub-properties and sub-methods:
            - ``.bitand()``: The ``bit.bitand()`` function.
            - ``.bitor()``: The ``bit.bitor()`` function.
            - ``.bitxor()``: The ``bit.bitxor()`` function.
            - ``.clear()``: The ``bit.clear()`` function.
            - ``.get()``: The ``bit.get()`` function.
            - ``.getfield()``: The ``bit.getfield()`` function.
            - ``.set()``: The ``bit.set()`` function.
            - ``.setfield()``: The ``bit.setfield()`` function.
            - ``.test()``: The ``bit.test()`` function.
            - ``.toggle()``: The ``bit.toggle()`` function.
        """
        return self._bit

    @property
    def buffer_var(self) -> Dict[str, Buffervar]:
        """Return the ``bufferVar`` command tree.

        Info:
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Sub-properties and sub-methods:
            - ``.appendmode``: The ``bufferVar.appendmode`` attribute.
            - ``.cachemode``: The ``bufferVar.cachemode`` attribute.
            - ``.capacity``: The ``bufferVar.capacity`` attribute.
            - ``.clear()``: The ``bufferVar.clear()`` function.
            - ``.clearcache()``: The ``bufferVar.clearcache()`` function.
            - ``.columnname``: The ``bufferVar.columnname`` command tree.
            - ``.fillcount``: The ``bufferVar.fillcount`` attribute.
            - ``.fillmode``: The ``bufferVar.fillmode`` attribute.
            - ``.fractionalseconds``: The ``bufferVar.fractionalseconds[N]`` attribute.
            - ``.measurefunctions``: The ``bufferVar.measurefunctions[N]`` attribute.
            - ``.measureranges``: The ``bufferVar.measureranges[N]`` attribute.
            - ``.n``: The ``bufferVar.n`` attribute.
            - ``.readings``: The ``bufferVar.readings[N]`` attribute.
            - ``.seconds``: The ``bufferVar.seconds[N]`` attribute.
            - ``.sourcefunctions``: The ``bufferVar.sourcefunctions[N]`` attribute.
            - ``.sourceoutputstates``: The ``bufferVar.sourceoutputstates[N]`` attribute.
            - ``.sourceranges``: The ``bufferVar.sourceranges[N]`` attribute.
            - ``.sourcevalues``: The ``bufferVar.sourcevalues[N]`` attribute.
            - ``.statuses``: The ``bufferVar.statuses[N]`` attribute.
            - ``.timestampresolution``: The ``bufferVar.timestampresolution`` attribute.
            - ``.timestamps``: The ``bufferVar.timestamps[N]`` attribute.
        """
        return self._buffer_var

    @property
    def dataqueue(self) -> Dataqueue:
        """Return the ``dataqueue`` command tree.

        Constants:
            - ``.CAPACITY``: The maximum number of entries that you can store in the data queue.

        Sub-properties and sub-methods:
            - ``.add()``: The ``dataqueue.add()`` function.
            - ``.clear()``: The ``dataqueue.clear()`` function.
            - ``.count``: The ``dataqueue.count`` attribute.
            - ``.next()``: The ``dataqueue.next()`` function.
        """
        return self._dataqueue

    @property
    def digio(self) -> Digio:
        """Return the ``digio`` command tree.

        Sub-properties and sub-methods:
            - ``.readbit()``: The ``digio.readbit()`` function.
            - ``.readport()``: The ``digio.readport()`` function.
            - ``.trigger``: The ``digio.trigger[N]`` command tree.
            - ``.writebit()``: The ``digio.writebit()`` function.
            - ``.writeport()``: The ``digio.writeport()`` function.
            - ``.writeprotect``: The ``digio.writeprotect`` attribute.
        """
        return self._digio

    @property
    def eventlog(self) -> Eventlog:
        """Return the ``eventlog`` command tree.

        Sub-properties and sub-methods:
            - ``.clear()``: The ``eventlog.clear()`` function.
            - ``.count``: The ``eventlog.count`` attribute.
            - ``.next()``: The ``eventlog.next()`` function.
        """
        return self._eventlog

    @property
    def firmware(self) -> Firmware:
        """Return the ``firmware`` command tree.

        Sub-properties and sub-methods:
            - ``.load()``: The ``firmware.load()`` function.
            - ``.update()``: The ``firmware.update()`` function.
        """
        return self._firmware

    @property
    def format(self) -> Format:
        """Return the ``format`` command tree.

        Sub-properties and sub-methods:
            - ``.asciiprecision``: The ``format.asciiprecision`` attribute.
            - ``.byteorder``: The ``format.byteorder`` attribute.
            - ``.data``: The ``format.data`` attribute.
        """
        return self._format

    @property
    def fs(self) -> Fs:
        """Return the ``fs`` command tree.

        Sub-properties and sub-methods:
            - ``.cwd()``: The ``fs.cwd()`` function.
            - ``.is_dir()``: The ``fs.is_dir()`` function.
            - ``.is_file()``: The ``fs.is_file()`` function.
            - ``.mkdir()``: The ``fs.mkdir()`` function.
            - ``.readdir()``: The ``fs.readdir()`` function.
            - ``.rmdir()``: The ``fs.rmdir()`` function.
        """
        return self._fs

    @property
    def io(self) -> Io:
        """Return the ``io`` command tree.

        Sub-properties and sub-methods:
            - ``.close()``: The ``io.close()`` function.
            - ``.flush()``: The ``io.flush()`` function.
            - ``.input()``: The ``io.input()`` function.
            - ``.open()``: The ``io.open()`` function.
            - ``.output()``: The ``io.output()`` function.
            - ``.read()``: The ``io.read()`` function.
            - ``.type()``: The ``io.type()`` function.
            - ``.write()``: The ``io.write()`` function.
        """
        return self._io

    @property
    def lan(self) -> Lan:
        """Return the ``lan`` command tree.

        Sub-properties and sub-methods:
            - ``.applysettings()``: The ``lan.applysettings()`` function.
            - ``.autoconnect``: The ``lan.autoconnect`` attribute.
            - ``.config``: The ``lan.config`` command tree.
            - ``.enable``: The ``lan.enable`` attribute.
            - ``.hislip``: The ``lan.hislip`` command tree.
            - ``.identify``: The ``lan.identify`` attribute.
            - ``.ipconfig()``: The ``lan.ipconfig()`` function.
            - ``.linktimeout``: The ``lan.linktimeout`` attribute.
            - ``.nagle``: The ``lan.nagle`` attribute.
            - ``.rawsocket``: The ``lan.rawsocket`` command tree.
            - ``.reset()``: The ``lan.reset()`` function.
            - ``.restoredefaults()``: The ``lan.restoredefaults()`` function.
            - ``.status``: The ``lan.status`` command tree.
            - ``.telnet``: The ``lan.telnet`` command tree.
        """
        return self._lan

    @property
    def localnode(self) -> Localnode:
        """Return the ``localnode`` command tree.

        Sub-properties and sub-methods:
            - ``.license``: The ``localnode.license`` attribute.
            - ``.linefreq``: The ``localnode.linefreq`` attribute.
            - ``.manufacturer``: The ``localnode.manufacturer`` attribute.
            - ``.model``: The ``localnode.model`` attribute.
            - ``.prompts``: The ``localnode.prompts`` attribute.
            - ``.prompts4882``: The ``localnode.prompts4882`` attribute.
            - ``.serialno``: The ``localnode.serialno`` attribute.
            - ``.showerrors``: The ``localnode.showerrors`` attribute.
            - ``.version``: The ``localnode.version`` attribute.
        """
        return self._localnode

    @property
    def node(self) -> Dict[int, NodeItem]:
        """Return the ``node[N]`` command tree.

        Info:
            - ``N``, the node number of the specified instrument.

        Sub-properties and sub-methods:
            - ``.execute()``: The ``node[N].execute()`` function.
            - ``.getglobal()``: The ``node[N].getglobal()`` function.
            - ``.setglobal()``: The ``node[N].setglobal()`` function.
        """
        return self._node

    @property
    def os(self) -> Os:
        """Return the ``os`` command tree.

        Sub-properties and sub-methods:
            - ``.clock()``: The ``os.clock()`` function.
            - ``.remove()``: The ``os.remove()`` function.
            - ``.rename()``: The ``os.rename()`` function.
            - ``.time()``: The ``os.time()`` function.
        """
        return self._os

    @property
    def script(self) -> Script:
        """Return the ``script`` command tree.

        Sub-properties and sub-methods:
            - ``.delete()``: The ``script.delete()`` function.
            - ``.load()``: The ``script.load()`` function.
            - ``.new()``: The ``script.new()`` function.
            - ``.newautorun()``: The ``script.newautorun()`` function.
            - ``.restore()``: The ``script.restore()`` function.
        """
        return self._script

    @property
    def script_var(self) -> Dict[str, Scriptvar]:
        """Return the ``scriptVar`` command tree.

        Info:
            - ``scriptVar``, the name of the variable that references the script.

        Sub-properties and sub-methods:
            - ``.autorun``: The ``scriptVar.autorun`` attribute.
            - ``.list()``: The ``scriptVar.list()`` function.
            - ``.name``: The ``scriptVar.name`` attribute.
            - ``.run()``: The ``scriptVar.run()`` function.
            - ``.save()``: The ``scriptVar.save()`` function.
            - ``.source``: The ``scriptVar.source`` attribute.
        """
        return self._script_var

    @property
    def slot(self) -> Dict[int, SlotItem]:
        """Return the ``slot[Z]`` command tree.

        Info:
            - ``Z``, the module slot number.

        Sub-properties and sub-methods:
            - ``.firmware``: The ``slot[Z].firmware`` command tree.
            - ``.license``: The ``slot[Z].license`` attribute.
            - ``.manufacturer``: The ``slot[Z].manufacturer`` attribute.
            - ``.model``: The ``slot[Z].model`` attribute.
            - ``.psu``: The ``slot[Z].psu[X]`` command tree.
            - ``.serialno``: The ``slot[Z].serialno`` attribute.
            - ``.smu``: The ``slot[Z].smu[X]`` command tree.
            - ``.status``: The ``slot[Z].status`` command tree.
            - ``.trigger``: The ``slot[Z].trigger`` command tree.
            - ``.version``: The ``slot[Z].version`` attribute.
        """
        return self._slot

    @property
    def status(self) -> Status:
        """Return the ``status`` command tree.

        Sub-properties and sub-methods:
            - ``.condition``: The ``status.condition`` attribute.
            - ``.node_enable``: The ``status.node_enable`` attribute.
            - ``.node_event``: The ``status.node_event`` attribute.
            - ``.operation``: The ``status.operation`` command tree.
            - ``.request_enable``: The ``status.request_enable`` attribute.
            - ``.request_event``: The ``status.request_event`` attribute.
            - ``.reset()``: The ``status.reset()`` function.
            - ``.standard``: The ``status.standard`` command tree.
            - ``.system``: The ``status.system`` command tree.
            - ``.system2``: The ``status.system2`` command tree.
            - ``.system3``: The ``status.system3`` command tree.
            - ``.system4``: The ``status.system4`` command tree.
            - ``.system5``: The ``status.system5`` command tree.
        """
        return self._status

    @property
    def timer(self) -> Timer:
        """Return the ``timer`` command tree.

        Sub-properties and sub-methods:
            - ``.cleartime()``: The ``timer.cleartime()`` function.
            - ``.measure``: The ``timer.measure`` command tree.
            - ``.readtime()``: The ``timer.readtime()`` function.
        """
        return self._timer

    @property
    def trigger(self) -> Trigger:
        """Return the ``trigger`` command tree.

        Constants:
            - ``.EVENT_ID``: The command interface trigger event number.
            - ``.EVENT_NONE``: This trigger event ID is never generated. It is used to disconnect
              stimulus inputs.

        Sub-properties and sub-methods:
            - ``.clear()``: The ``trigger.clear()`` function.
            - ``.detector``: The ``trigger.detector[N]`` command tree.
            - ``.generator``: The ``trigger.generator[N]`` command tree.
            - ``.timer``: The ``trigger.timer[N]`` command tree.
            - ``.wait()``: The ``trigger.wait()`` function.
        """
        return self._trigger

    @property
    def tsplink(self) -> Tsplink:
        """Return the ``tsplink`` command tree.

        Sub-properties and sub-methods:
            - ``.group``: The ``tsplink.group`` attribute.
            - ``.master``: The ``tsplink.master`` attribute.
            - ``.node``: The ``tsplink.node`` attribute.
            - ``.readbit()``: The ``tsplink.readbit()`` function.
            - ``.readport()``: The ``tsplink.readport()`` function.
            - ``.reset()``: The ``tsplink.reset()`` function.
            - ``.state``: The ``tsplink.state`` attribute.
            - ``.trigger``: The ``tsplink.trigger[N]`` command tree.
            - ``.writebit()``: The ``tsplink.writebit()`` function.
            - ``.writeport()``: The ``tsplink.writeport()`` function.
            - ``.writeprotect``: The ``tsplink.writeprotect`` attribute.
        """
        return self._tsplink

    @property
    def tspnet(self) -> Tspnet:
        """Return the ``tspnet`` command tree.

        Sub-properties and sub-methods:
            - ``.clear()``: The ``tspnet.clear()`` function.
            - ``.connect()``: The ``tspnet.connect()`` function.
            - ``.disconnect()``: The ``tspnet.disconnect()`` function.
            - ``.execute()``: The ``tspnet.execute()`` function.
            - ``.idn()``: The ``tspnet.idn()`` function.
            - ``.read()``: The ``tspnet.read()`` function.
            - ``.readavailable()``: The ``tspnet.readavailable()`` function.
            - ``.reset()``: The ``tspnet.reset()`` function.
            - ``.termination()``: The ``tspnet.termination()`` function.
            - ``.timeout``: The ``tspnet.timeout`` attribute.
            - ``.tsp``: The ``tspnet.tsp`` command tree.
            - ``.write()``: The ``tspnet.write()`` function.
        """
        return self._tspnet

    @property
    def usbtmc(self) -> Usbtmc:
        """Return the ``usbtmc`` command tree.

        Sub-properties and sub-methods:
            - ``.access``: The ``usbtmc.access`` attribute.
        """
        return self._usbtmc

    @property
    def user(self) -> User:
        """Return the ``user`` command tree.

        Sub-properties and sub-methods:
            - ``.password``: The ``user.password`` command tree.
        """
        return self._user

    @property
    def userstring(self) -> Userstring:
        """Return the ``userstring`` command tree.

        Sub-properties and sub-methods:
            - ``.add()``: The ``userstring.add()`` function.
            - ``.delete()``: The ``userstring.delete()`` function.
            - ``.get()``: The ``userstring.get()`` function.
        """
        return self._userstring

    def delay(self, seconds: int) -> None:
        """Run the ``delay()`` function.

        Description:
            - This function delays the execution of the commands that follow it.

        TSP Syntax:
            ```
            - delay()
            ```

        Args:
            seconds: The number of seconds to delay.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"delay({seconds})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``delay()`` function."
            raise NoDeviceProvidedError(msg) from error

    def exit(self) -> None:
        """Run the ``exit()`` function.

        Description:
            - This function stops a script that is presently running

        TSP Syntax:
            ```
            - exit()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                "exit()"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``exit()`` function."
            raise NoDeviceProvidedError(msg) from error

    def gettimezone(self) -> str:
        """Run the ``gettimezone()`` function.

        Description:
            - This function retrieves the local time zone.

        TSP Syntax:
            ```
            - gettimezone()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                "print(gettimezone())"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``gettimezone()`` function."
            raise NoDeviceProvidedError(msg) from error

    def login(self, password: str) -> None:
        """Run the ``login()`` function.

        Description:
            - This command logs a user in to the mainframe and allows remote access.

        TSP Syntax:
            ```
            - login()
            ```

        Args:
            password: A string that represents the login password. The default value is no password.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'login("{password}")'
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``login()`` function."
            raise NoDeviceProvidedError(msg) from error

    def logout(self) -> None:
        """Run the ``logout()`` function.

        Description:
            - This function logs a user out of the mainframe.

        TSP Syntax:
            ```
            - logout()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                "logout()"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``logout()`` function."
            raise NoDeviceProvidedError(msg) from error

    def makegetter(self, table: str, attribute_name: str) -> str:
        """Run the ``makegetter()`` function.

        Description:
            - This function creates a function to get the value of an attribute.

        TSP Syntax:
            ```
            - makegetter()
            ```

        Args:
            table: Read-only table where the attribute is located.
            attribute_name: A string representing the name of the attribute.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print(makegetter({table}, "{attribute_name}"))'
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``makegetter()`` function."
            raise NoDeviceProvidedError(msg) from error

    def makesetter(self, table: str, attribute_name: str) -> str:
        """Run the ``makesetter()`` function.

        Description:
            - This function creates a function that, when called, sets the value of an attribute.

        TSP Syntax:
            ```
            - makesetter()
            ```

        Args:
            table: Read-only table where the attribute is located.
            attribute_name: The string name of the attribute.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print(makesetter({table}, "{attribute_name}"))'
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``makesetter()`` function."
            raise NoDeviceProvidedError(msg) from error

    def opc(self) -> None:
        """Run the ``opc()`` function.

        Description:
            - This function sets the operation complete status bit when all overlapped commands are
              completed.

        TSP Syntax:
            ```
            - opc()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                "opc()"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``opc()`` function."
            raise NoDeviceProvidedError(msg) from error

    def print(self, value: str) -> None:
        """Run the ``print()`` function.

        Description:
            - This function generates a response message.

        TSP Syntax:
            ```
            - print()
            ```

        Args:
            value: The first argument to output.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"print({value})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``print()`` function."
            raise NoDeviceProvidedError(msg) from error

    def printbuffer(self, start_index: int, end_index: int, buffer_var: str) -> str:
        """Run the ``printbuffer()`` function.

        Description:
            - This function prints data from tables or reading buffer subtables.

        TSP Syntax:
            ```
            - printbuffer()
            ```

        Args:
            start_index: Beginning index of the buffer to print; this must be more than one and less
                than endIndex.
            end_index: Ending index of the buffer to print; this must be more than startIndex and
                less than the index of the last entry in the tables.
            buffer_var: First table or reading buffer subtable to print.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"printbuffer({start_index}, {end_index}, {buffer_var})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``printbuffer()`` function."
            raise NoDeviceProvidedError(msg) from error

    def printnumber(self, value: str) -> str:
        """Run the ``printnumber()`` function.

        Description:
            - This function prints numbers using the configured format.

        TSP Syntax:
            ```
            - printnumber()
            ```

        Args:
            value: First value to print in the configured format.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"printnumber({value})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``printnumber()`` function."
            raise NoDeviceProvidedError(msg) from error

    def reset(self, system: Optional[str] = None) -> None:
        """Run the ``reset()`` function.

        Description:
            - This function resets commands on all slots and channels to their default settings.

        TSP Syntax:
            ```
            - reset()
            ```

        Args:
            system (optional): What to reset.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (system,) if x is not None)
            self._device.write(  # type: ignore[union-attr]
                f"reset({function_args})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``reset()`` function."
            raise NoDeviceProvidedError(msg) from error

    def settime(self, seconds: str, fractional: Optional[int] = None) -> None:
        """Run the ``settime()`` function.

        Description:
            - This function sets the present time and the real time clock.

        TSP Syntax:
            ```
            - settime()
            ```

        Args:
            seconds: The time in seconds since January 1, 1970, UTC.
            fractional (optional): The number of fractional seconds.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    seconds,
                    fractional,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"settime({function_args})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``settime()`` function."
            raise NoDeviceProvidedError(msg) from error

    def settimezone(
        self,
        offset: str,
        dst_offset: Optional[str] = None,
        dst_start: Optional[str] = None,
        dst_end: Optional[str] = None,
    ) -> None:
        """Run the ``settimezone()`` function.

        Description:
            - This function sets the local time zone.

        TSP Syntax:
            ```
            - settimezone()
            ```

        Args:
            offset: String representing offset from UTC.
            dst_offset (optional): String representing the daylight savings offset from UTC.
            dst_start (optional): String representing when daylight savings time starts.
            dst_end (optional): String representing when daylight savings time ends.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{offset}"',
                    f'"{dst_offset}"' if dst_offset is not None else None,
                    f'"{dst_start}"' if dst_start is not None else None,
                    f'"{dst_end}"' if dst_end is not None else None,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"settimezone({function_args})"
            )
        except AttributeError as error:
            msg = "No TSPControl object was provided, unable to run the ``settimezone()`` function."
            raise NoDeviceProvidedError(msg) from error

    def waitcomplete(self, group: Optional[str] = None) -> None:
        """Run the ``waitcomplete()`` function.

        Description:
            - This function waits for all previously started overlapped commands to complete.

        TSP Syntax:
            ```
            - waitcomplete()
            ```

        Args:
            group (optional): Specifies which TSP-Link group on which to wait.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (group,) if x is not None)
            self._device.write(  # type: ignore[union-attr]
                f"waitcomplete({function_args})"
            )
        except AttributeError as error:
            msg = (
                "No TSPControl object was provided, unable to run the ``waitcomplete()`` function."
            )
            raise NoDeviceProvidedError(msg) from error


class MP5103Mixin:
    """A mixin that provides access to the MP5103 commands and constants.

    Properties:
        - ``.command_argument_constants``: The MP5103 command argument constants.
        - ``.commands``: The MP5103 commands.
    """

    @cached_property
    def command_argument_constants(self) -> MP5103CommandConstants:  # pylint: disable=no-self-use
        """Return the MP5103 command argument constants.

        This provides access to all the string constants which can be used as arguments for MP5103
        commands.
        """
        return MP5103CommandConstants()

    @cached_property
    def commands(self) -> MP5103Commands:
        """Return the MP5103 commands.

        This provides access to all the commands for the MP5103 device. See the documentation of
        each sub-property for more usage information.

        Sub-properties and sub-methods:
            - ``.bit``: The ``bit`` command tree.
            - ``.buffer_var``: The ``bufferVar`` command tree.
            - ``.dataqueue``: The ``dataqueue`` command tree.
            - ``.delay()``: The ``delay()`` function.
            - ``.digio``: The ``digio`` command tree.
            - ``.eventlog``: The ``eventlog`` command tree.
            - ``.exit()``: The ``exit()`` function.
            - ``.firmware``: The ``firmware`` command tree.
            - ``.format``: The ``format`` command tree.
            - ``.fs``: The ``fs`` command tree.
            - ``.gettimezone()``: The ``gettimezone()`` function.
            - ``.io``: The ``io`` command tree.
            - ``.lan``: The ``lan`` command tree.
            - ``.localnode``: The ``localnode`` command tree.
            - ``.login()``: The ``login()`` function.
            - ``.logout()``: The ``logout()`` function.
            - ``.makegetter()``: The ``makegetter()`` function.
            - ``.makesetter()``: The ``makesetter()`` function.
            - ``.node``: The ``node[N]`` command tree.
            - ``.opc()``: The ``opc()`` function.
            - ``.os``: The ``os`` command tree.
            - ``.print()``: The ``print()`` function.
            - ``.printbuffer()``: The ``printbuffer()`` function.
            - ``.printnumber()``: The ``printnumber()`` function.
            - ``.reset()``: The ``reset()`` function.
            - ``.script_var``: The ``scriptVar`` command tree.
            - ``.script``: The ``script`` command tree.
            - ``.settime()``: The ``settime()`` function.
            - ``.settimezone()``: The ``settimezone()`` function.
            - ``.slot``: The ``slot[Z]`` command tree.
            - ``.status``: The ``status`` command tree.
            - ``.timer``: The ``timer`` command tree.
            - ``.trigger``: The ``trigger`` command tree.
            - ``.tsplink``: The ``tsplink`` command tree.
            - ``.tspnet``: The ``tspnet`` command tree.
            - ``.usbtmc``: The ``usbtmc`` command tree.
            - ``.user``: The ``user`` command tree.
            - ``.userstring``: The ``userstring`` command tree.
            - ``.waitcomplete()``: The ``waitcomplete()`` function.
        """
        device = self if isinstance(self, TSPControl) else None
        return MP5103Commands(device)
